from collections import namedtuple
from typing import Optional

from iglovikov_helper_functions.dl.pytorch.utils import rename_layers
from timm import create_model as timm_create_model
from torch import nn
from torch.utils import model_zoo

model = namedtuple("model", ["url", "model"])

models = {
    "swsl_resnext50_32x4d": model(
        model=timm_create_model("swsl_resnext50_32x4d", pretrained=False, num_classes=4),
        url="https://github.com/ternaus/check_orientation/releases/download/v0.0.3/2020-11-16_resnext50_32x4d.zip",
    ),
}


def create_model(model_name: str, activation: Optional[str] = "softmax") -> nn.Module:
    model = models[model_name].model
    state_dict = model_zoo.load_url(models[model_name].url, progress=True, map_location="cpu")["state_dict"]
    state_dict = rename_layers(state_dict, {"model.": ""})
    model.load_state_dict(state_dict)

    if activation == "softmax":
        return nn.Sequential(model, nn.Softmax(dim=1))

    return model



import argparse
from pathlib import Path
from typing import Any, Dict, List, Optional

import albumentations as albu
import numpy as np
import torch
import torch.nn.parallel
import torch.utils.data
import torch.utils.data.distributed
import yaml
from albumentations.core.serialization import from_dict
from iglovikov_helper_functions.config_parsing.utils import object_from_dict
from iglovikov_helper_functions.dl.pytorch.utils import (
    state_dict_from_disk,
    tensor_from_rgb_image,
)
from iglovikov_helper_functions.utils.image_utils import load_rgb
from torch import nn
from torch.utils.data import Dataset
from torch.utils.data.distributed import DistributedSampler
from tqdm import tqdm


def get_args():
    parser = argparse.ArgumentParser()
    arg = parser.add_argument
    arg("-i", "--input_path", type=Path, help="Path with images.", required=True)
    arg("-c", "--config_path", type=Path, help="Path to config.", required=True)
    arg("-o", "--output_path", type=Path, help="Path to save masks.", required=True)
    arg("-b", "--batch_size", type=int, help="batch_size", default=1)
    arg("-j", "--num_workers", type=int, help="num_workers", default=4)
    arg("-w", "--weight_path", type=str, help="Path to weights.", required=True)
    arg("--world_size", default=-1, type=int, help="number of nodes for distributed training")
    arg("--local_rank", default=-1, type=int, help="node rank for distributed training")
    arg("--fp16", action="store_true", help="Use fp6")
    return parser.parse_args()


class InferenceDataset(Dataset):
    def __init__(self, file_paths: List[Path], transform: albu.Compose) -> None:
        self.file_paths = file_paths
        self.transform = transform

    def __len__(self) -> int:
        return len(self.file_paths)

    def __getitem__(self, idx: int) -> Optional[Dict[str, Any]]:
        image_path = self.file_paths[idx]

        image = load_rgb(image_path)

        image = self.transform(image=image)["image"]

        return {"torched_image": tensor_from_rgb_image(image), "image_path": str(image_path)}


def main():
    args = get_args()
    torch.distributed.init_process_group(backend="nccl")

    with open(args.config_path) as f:
        hparams = yaml.load(f, Loader=yaml.SafeLoader)

    hparams.update(
        {
            "local_rank": args.local_rank,
            "fp16": args.fp16,
        }
    )

    args.output_path.mkdir(parents=True, exist_ok=True)
    hparams["output_path"] = args.output_path

    device = torch.device("cuda", args.local_rank)  # pylint: disable=E1101

    model = object_from_dict(hparams["model"])

    corrections: Dict[str, str] = {"model.": ""}
    state_dict = state_dict_from_disk(file_path=args.weight_path, rename_in_layers=corrections)
    model.load_state_dict(state_dict)

    model = nn.Sequential(model, nn.Softmax(dim=1))
    model = model.to(device)

    if args.fp16:
        model = model.half()

    model = torch.nn.parallel.DistributedDataParallel(
        model, device_ids=[args.local_rank], output_device=args.local_rank
    )

    file_paths = []

    for regexp in ["*.jpg", "*.png", "*.jpeg", "*.JPG"]:
        file_paths += sorted(args.input_path.rglob(regexp))

    # Filter file paths for which we already have predictions
    file_paths = [x for x in file_paths if not (args.output_path / x.parent.name / f"{x.stem}.txt").exists()]

    dataset = InferenceDataset(file_paths, transform=from_dict(hparams["test_aug"]))

    sampler = DistributedSampler(dataset, shuffle=False)

    dataloader = torch.utils.data.DataLoader(
        dataset,
        batch_size=args.batch_size,
        num_workers=args.num_workers,
        pin_memory=True,
        shuffle=False,
        drop_last=False,
        sampler=sampler,
    )

    predict(dataloader, model, hparams, device)


def predict(dataloader, model, hparams, device):
    model.eval()

    if hparams["local_rank"] == 0:
        loader = tqdm(dataloader)
    else:
        loader = dataloader

    with torch.no_grad():
        for batch in loader:
            torched_images = batch["torched_image"]  # images that are rescaled and padded

            if hparams["fp16"]:
                torched_images = torched_images.half()

            image_paths = batch["image_path"]

            batch_size = torched_images.shape[0]

            predictions = model(torched_images.to(device))

            for batch_id in range(batch_size):
                file_id = Path(image_paths[batch_id]).stem
                folder_name = Path(image_paths[batch_id]).parent.name

                prob = predictions[batch_id].cpu().numpy().astype(np.float16)

                (hparams["output_path"] / folder_name).mkdir(exist_ok=True, parents=True)

                with open(str(hparams["output_path"] / folder_name / f"{file_id}.txt"), "w") as f:
                    f.write(str(prob.tolist()))


if __name__ == "__main__":
    main()



import argparse
import os
from pathlib import Path

import pytorch_lightning as pl
import torch
import yaml
from addict import Dict as Adict
from albumentations.core.serialization import from_dict
from iglovikov_helper_functions.config_parsing.utils import object_from_dict
from pytorch_lightning.loggers import WandbLogger
from torch.utils.data import DataLoader

from check_orientation.dataloaders import ClassificationDataset

train_image_path = Path(os.environ["TRAIN_IMAGE_PATH"])
val_image_path = Path(os.environ["VAL_IMAGE_PATH"])


def get_args():
    parser = argparse.ArgumentParser()
    arg = parser.add_argument
    arg("-c", "--config_path", type=Path, help="Path to the config.", required=True)
    return parser.parse_args()


class CheckOrientation(pl.LightningModule):
    def __init__(self, hparams):
        super().__init__()
        self.hparams = hparams

        self.model = object_from_dict(self.hparams.model)
        self.loss = object_from_dict(self.hparams.loss)
        self.train_accuracy = pl.metrics.Accuracy()
        self.val_accuracy = pl.metrics.Accuracy()

    def forward(self, batch):
        return self.model(batch)

    def setup(self, stage=0):  # pylint: disable=W0613
        self.train_image_paths = sorted(train_image_path.glob("*.jpg"))
        self.val_image_paths = sorted(val_image_path.glob("*.jpg"))

        print("Len train samples = ", len(self.train_image_paths))
        print("Len val samples = ", len(self.val_image_paths))

    def train_dataloader(self):
        train_aug = from_dict(self.hparams.train_aug)

        if "epoch_length" not in self.hparams.train_parameters:
            epoch_length = None
        else:
            epoch_length = self.hparams.train_parameters.epoch_length

        result = DataLoader(
            ClassificationDataset(self.train_image_paths, train_aug, epoch_length),
            batch_size=self.hparams.train_parameters.batch_size,
            num_workers=self.hparams.num_workers,
            shuffle=True,
            pin_memory=True,
            drop_last=True,
        )

        print("Train dataloader = ", len(result))
        return result

    def val_dataloader(self):
        val_aug = from_dict(self.hparams.val_aug)

        result = DataLoader(
            ClassificationDataset(self.val_image_paths, val_aug, length=None),
            batch_size=self.hparams.val_parameters.batch_size,
            num_workers=self.hparams.num_workers,
            shuffle=False,
            pin_memory=True,
            drop_last=False,
        )

        print("Val dataloader = ", len(result))
        return result

    def configure_optimizers(self):
        optimizer = object_from_dict(
            self.hparams["optimizer"],
            params=[x for x in self.model.parameters() if x.requires_grad],
        )

        scheduler = object_from_dict(self.hparams.scheduler, optimizer=optimizer)
        self.optimizers = [optimizer]

        return self.optimizers, [scheduler]

    def training_step(self, batch, batch_idx):  # pylint: disable=W0613
        features = batch["features"]
        targets = batch["targets"]

        logits = self.forward(features)

        loss = self.loss(logits, targets)

        self.log("lr", self._get_current_lr(), on_step=True, on_epoch=False, prog_bar=True, logger=True)
        self.log("train_loss", loss, on_step=True, on_epoch=False, prog_bar=False, logger=True)
        self.log(
            "train_acc", self.train_accuracy(logits, targets), on_step=True, on_epoch=True, prog_bar=True, logger=True
        )

        return loss

    def _get_current_lr(self) -> torch.Tensor:
        lr = [x["lr"] for x in self.optimizers[0].param_groups][0]  # type: ignore
        return torch.Tensor([lr])[0].cuda()

    def validation_step(self, batch, batch_id):  # pylint: disable=W0613
        features = batch["features"]
        targets = batch["targets"].long()

        logits = self.forward(features)
        loss = self.loss(logits, targets)

        self.log("val_loss", loss, on_step=False, on_epoch=True, prog_bar=False, logger=True)
        self.log(
            "val_acc", self.val_accuracy(logits, targets), on_step=False, on_epoch=True, prog_bar=False, logger=True
        )


def main():
    args = get_args()

    with open(args.config_path) as f:
        hparams = Adict(yaml.load(f, Loader=yaml.SafeLoader))

    pl.trainer.seed_everything(hparams.seed)

    pipeline = CheckOrientation(hparams)

    Path(hparams.checkpoint_callback.filepath).mkdir(exist_ok=True, parents=True)

    trainer = object_from_dict(
        hparams.trainer,
        logger=WandbLogger(hparams.experiment_name),
        checkpoint_callback=object_from_dict(hparams.checkpoint_callback),
    )

    trainer.fit(pipeline)


if __name__ == "__main__":
    main()



import random
from pathlib import Path
from typing import Any, Dict, List

import albumentations as albu
import numpy as np
from iglovikov_helper_functions.dl.pytorch.utils import tensor_from_rgb_image
from iglovikov_helper_functions.utils.image_utils import load_rgb
from torch.utils.data import Dataset


class ClassificationDataset(Dataset):
    def __init__(
        self,
        image_paths: List[Path],
        transform: albu.Compose,
        length: int = None,
    ) -> None:
        self.image_paths = image_paths
        self.transform = transform

        if length is None:
            self.length = len(self.image_paths)
        else:
            self.length = length

    def __len__(self) -> int:
        return self.length

    def __getitem__(self, idx: int) -> Dict[str, Any]:
        idx = idx % len(self.image_paths)

        image_path = self.image_paths[idx]

        image = load_rgb(image_path, lib="cv2")

        # apply augmentations
        image = self.transform(image=image)["image"]

        orientation = random.randint(0, 3)
        image = np.ascontiguousarray(np.rot90(image, orientation))

        return {"image_id": image_path.stem, "features": tensor_from_rgb_image(image), "targets": orientation}
